@@ -14,6 +14,83 @@ from xml.dom import minidom
 __all__ = ['EventPlanned', 'EventUnplanned', 'EventMedia', 'SignInventory', 'SignStatus',
            'RouteInventory', 'RouteStatus', 'RwisInventory', 'RwisStatus']
 
+
+county_lookup = {
+    'ADAMS': {'cntyCode': 1, 'region': 'NC', 'fips': 55001, 'district': 4, 'regionOffice': 'RAP'},
+    'ASHLAND': {'cntyCode': 2, 'region': 'NW', 'fips': 55003, 'district': 8, 'regionOffice': 'SUP'},
+    'BARRON': {'cntyCode': 3, 'region': 'NW', 'fips': 55005, 'district': 8, 'regionOffice': 'SUP'},
+    'BAYFIELD': {'cntyCode': 4, 'region': 'NW', 'fips': 55007, 'district': 8, 'regionOffice': 'SUP'},
+    'BROWN': {'cntyCode': 5, 'region': 'NE', 'fips': 55009, 'district': 3, 'regionOffice': 'GRB'},
+    'BUFFALO': {'cntyCode': 6, 'region': 'NW', 'fips': 55011, 'district': 5, 'regionOffice': 'EAU'},
+    'BURNETT': {'cntyCode': 7, 'region': 'NW', 'fips': 55013, 'district': 8, 'regionOffice': 'SUP'},
+    'CALUMET': {'cntyCode': 8, 'region': 'NE', 'fips': 55015, 'district': 3, 'regionOffice': 'GRB'},
+    'CHIPPEWA': {'cntyCode': 9, 'region': 'NW', 'fips': 55017, 'district': 6, 'regionOffice': 'EAU'},
+    'CLARK': {'cntyCode': 10, 'region': 'NW', 'fips': 55019, 'district': 6, 'regionOffice': 'EAU'},
+    'COLUMBIA': {'cntyCode': 11, 'region': 'SW', 'fips': 55021, 'district': 1, 'regionOffice': 'MAD'},
+    'CRAWFORD': {'cntyCode': 12, 'region': 'SW', 'fips': 55023, 'district': 5, 'regionOffice': 'LAC'},
+    'DANE': {'cntyCode': 13, 'region': 'SW', 'fips': 55025, 'district': 1, 'regionOffice': 'MAD'},
+    'DODGE': {'cntyCode': 14, 'region': 'SW', 'fips': 55027, 'district': 1, 'regionOffice': 'MAD'},
+    'DOOR': {'cntyCode': 15, 'region': 'NE', 'fips': 55029, 'district': 3, 'regionOffice': 'GRB'},
+    'DOUGLAS': {'cntyCode': 16, 'region': 'NW', 'fips': 55031, 'district': 8, 'regionOffice': 'SUP'},
+    'DUNN': {'cntyCode': 17, 'region': 'NW', 'fips': 55033, 'district': 6, 'regionOffice': 'EAU'},
+    'EAU CLAIRE': {'cntyCode': 18, 'region': 'NW', 'fips': 55035, 'district': 6, 'regionOffice': 'EAU'},
+    'FLORENCE': {'cntyCode': 19, 'region': 'NC', 'fips': 55037, 'district': 7, 'regionOffice': 'RHI'},
+    'FOND DU LAC': {'cntyCode': 20, 'region': 'NE', 'fips': 55039, 'district': 2, 'regionOffice': 'GRB'},
+    'FOREST': {'cntyCode': 21, 'region': 'NC', 'fips': 55041, 'district': 7, 'regionOffice': 'RHI'},
+    'GRANT': {'cntyCode': 22, 'region': 'SW', 'fips': 55043, 'district': 1, 'regionOffice': 'LAC'},
+    'GREEN': {'cntyCode': 23, 'region': 'SW', 'fips': 55045, 'district': 1, 'regionOffice': 'MAD'},
+    'GREEN LAKE': {'cntyCode': 24, 'region': 'NC', 'fips': 55047, 'district': 4, 'regionOffice': 'RAP'},
+    'IOWA': {'cntyCode': 25, 'region': 'SW', 'fips': 55049, 'district': 1, 'regionOffice': 'MAD'},
+    'IRON': {'cntyCode': 26, 'region': 'NC', 'fips': 55051, 'district': 7, 'regionOffice': 'RHI'},
+    'JACKSON': {'cntyCode': 27, 'region': 'NW', 'fips': 55053, 'district': 5, 'regionOffice': 'EAU'},
+    'JEFFERSON': {'cntyCode': 28, 'region': 'SW', 'fips': 55055, 'district': 1, 'regionOffice': 'MAD'},
+    'JUNEAU': {'cntyCode': 29, 'region': 'SW', 'fips': 55057, 'district': 4, 'regionOffice': 'LAC'},
+    'KENOSHA': {'cntyCode': 30, 'region': 'SE', 'fips': 55059, 'district': 2, 'regionOffice': 'WKE'},
+    'KEWAUNEE': {'cntyCode': 31, 'region': 'NE', 'fips': 55061, 'district': 3, 'regionOffice': 'GRB'},
+    'LA CROSSE': {'cntyCode': 32, 'region': 'SW', 'fips': 55063, 'district': 5, 'regionOffice': 'LAC'},
+    'LAFAYETTE': {'cntyCode': 33, 'region': 'SW', 'fips': 55065, 'district': 1, 'regionOffice': 'MAD'},
+    'LANGLADE': {'cntyCode': 34, 'region': 'NC', 'fips': 55067, 'district': 7, 'regionOffice': 'RHI'},
+    'LINCOLN': {'cntyCode': 35, 'region': 'NC', 'fips': 55069, 'district': 7, 'regionOffice': 'RHI'},
+    'MANITOWOC': {'cntyCode': 36, 'region': 'NE', 'fips': 55071, 'district': 3, 'regionOffice': 'GRB'},
+    'MARATHON': {'cntyCode': 37, 'region': 'NC', 'fips': 55073, 'district': 4, 'regionOffice': 'RAP'},
+    'MARINETTE': {'cntyCode': 38, 'region': 'NE', 'fips': 55075, 'district': 3, 'regionOffice': 'GRB'},
+    'MARQUETTE': {'cntyCode': 39, 'region': 'NC', 'fips': 55077, 'district': 4, 'regionOffice': 'RAP'},
+    'MILWAUKEE': {'cntyCode': 40, 'region': 'SE', 'fips': 55079, 'district': 2, 'regionOffice': 'WKE'},
+    'MONROE': {'cntyCode': 41, 'region': 'SW', 'fips': 55081, 'district': 5, 'regionOffice': 'LAC'},
+    'OCONTO': {'cntyCode': 42, 'region': 'NE', 'fips': 55083, 'district': 3, 'regionOffice': 'GRB'},
+    'ONEIDA': {'cntyCode': 43, 'region': 'NC', 'fips': 55085, 'district': 7, 'regionOffice': 'RHI'},
+    'OUTAGAMIE': {'cntyCode': 44, 'region': 'NE', 'fips': 55087, 'district': 3, 'regionOffice': 'GRB'},
+    'OZAUKEE': {'cntyCode': 45, 'region': 'SE', 'fips': 55089, 'district': 2, 'regionOffice': 'WKE'},
+    'PEPIN': {'cntyCode': 46, 'region': 'NW', 'fips': 55091, 'district': 6, 'regionOffice': 'EAU'},
+    'PIERCE': {'cntyCode': 47, 'region': 'NW', 'fips': 55093, 'district': 6, 'regionOffice': 'EAU'},
+    'POLK': {'cntyCode': 48, 'region': 'NW', 'fips': 55095, 'district': 8, 'regionOffice': 'SUP'},
+    'PORTAGE': {'cntyCode': 49, 'region': 'NC', 'fips': 55097, 'district': 4, 'regionOffice': 'RAP'},
+    'PRICE': {'cntyCode': 50, 'region': 'NC', 'fips': 55099, 'district': 7, 'regionOffice': 'RHI'},
+    'RACINE': {'cntyCode': 51, 'region': 'SE', 'fips': 55101, 'district': 2, 'regionOffice': 'WKE'},
+    'RICHLAND': {'cntyCode': 52, 'region': 'SW', 'fips': 55103, 'district': 5, 'regionOffice': 'LAC'},
+    'ROCK': {'cntyCode': 53, 'region': 'SW', 'fips': 55105, 'district': 1, 'regionOffice': 'MAD'},
+    'RUSK': {'cntyCode': 54, 'region': 'NW', 'fips': 55107, 'district': 8, 'regionOffice': 'SUP'},
+    'ST. CROIX': {'cntyCode': 55, 'region': 'NW', 'fips': 55109, 'district': 6, 'regionOffice': 'EAU'},
+    'SAUK': {'cntyCode': 56, 'region': 'SW', 'fips': 55111, 'district': 1, 'regionOffice': 'MAD'},
+    'SAWYER': {'cntyCode': 57, 'region': 'NW', 'fips': 55113, 'district': 8, 'regionOffice': 'SUP'},
+    'SHAWANO': {'cntyCode': 58, 'region': 'NC', 'fips': 55115, 'district': 3, 'regionOffice': 'RHI'},
+    'SHEBOYGAN': {'cntyCode': 59, 'region': 'NE', 'fips': 55117, 'district': 3, 'regionOffice': 'GRB'},
+    'TAYLOR': {'cntyCode': 60, 'region': 'NW', 'fips': 55119, 'district': 6, 'regionOffice': 'EAU'},
+    'TREMPEALEAU': {'cntyCode': 61, 'region': 'NW', 'fips': 55121, 'district': 5, 'regionOffice': 'EAU'},
+    'VERNON': {'cntyCode': 62, 'region': 'SW', 'fips': 55123, 'district': 5, 'regionOffice': 'LAC'},
+    'VILAS': {'cntyCode': 63, 'region': 'NC', 'fips': 55125, 'district': 7, 'regionOffice': 'RHI'},
+    'WALWORTH': {'cntyCode': 64, 'region': 'SE', 'fips': 55127, 'district': 2, 'regionOffice': 'WKE'},
+    'WASHBURN': {'cntyCode': 65, 'region': 'NW', 'fips': 55129, 'district': 8, 'regionOffice': 'SUP'},
+    'WASHINGTON': {'cntyCode': 66, 'region': 'SE', 'fips': 55131, 'district': 2, 'regionOffice': 'WKE'},
+    'WAUKESHA': {'cntyCode': 67, 'region': 'SE', 'fips': 55133, 'district': 2, 'regionOffice': 'WKE'},
+    'WAUPACA': {'cntyCode': 68, 'region': 'NC', 'fips': 55135, 'district': 4, 'regionOffice': 'RAP'},
+    'WAUSHARA': {'cntyCode': 69, 'region': 'NC', 'fips': 55137, 'district': 4, 'regionOffice': 'RAP'},
+    'WINNEBAGO': {'cntyCode': 70, 'region': 'NE', 'fips': 55139, 'district': 3, 'regionOffice': 'GRB'},
+    'WOOD': {'cntyCode': 71, 'region': 'NC', 'fips': 55141, 'district': 4, 'regionOffice': 'RAP'},
+    'MENOMINEE': {'cntyCode': 73, 'region': 'NC', 'fips': 55078, 'district': 3, 'regionOffice': 'RHI'}
+
+}
+
 #TODO make sql generator for cams and message signs
 
 # noinspection DuplicatedCode
@@ -33,7 +110,7 @@ def prettify_xml(elem):
     return reparsed.toprettyxml(indent="  ").replace('xsinoNamespaceSchemaLocation', 'xsi:noNamespaceSchemaLocation')
 
 
-def _get_org_info(el_name: str):
+def _get_org_info(el_name: str, cntr_id='', cntr_name=''):
     _org_info = ElementTree.Element(el_name)
 
     _org_id = ElementTree.Element('organization-id')
@@ -45,11 +122,11 @@ def _get_org_info(el_name: str):
     _org_info.append(_org_name)
 
     _org_center_id = ElementTree.Element('center-id')
-    _org_center_id.text = 'MILWAUKEE'
+    _org_center_id.text = cntr_id
     _org_info.append(_org_center_id)
 
     _org_center_name = ElementTree.Element('center-name')
-    _org_center_name.text = 'Milwaukee Center'
+    _org_center_name.text = cntr_name
     _org_info.append(_org_center_name)
 
     return _org_info
@@ -230,6 +307,7 @@ class _Location:
         self.direction: str = dta.get('direction', None)
         self.longitude: str = dta.get('longitude', None)
         self.latitude: str = dta.get('latitude', None)
+        self.county: str = dta.get('county', None)
 
     @property
     def json(self) -> dict:
@@ -237,7 +315,8 @@ class _Location:
             "roadwayName": self.roadway_name,
             "direction": self.direction,
             "latitude": self.latitude,
-            "longitude": self.longitude
+            "longitude": self.longitude,
+            "county": self.county,
         }
 
 
@@ -254,6 +333,7 @@ class _EventBase(Atms, _GeoJson):
 
     def __init__(self, data: dict):
         super().__init__(data)
+        # print(data)
         self._description: str = data['description']
         self._head_location: _Location = _Location(data['headlocation'])
         self._tail_location: _Location = _Location(data['taillocation'])
@@ -338,10 +418,22 @@ class EventUnplanned(_EventBase):
         # print(self.id)
         # print(json.dumps(self.json, indent=4))
 
+        # print(self._head_location.county.upper())
+
+        # print(county_lookup.get(self._head_location.county.upper(), None))
+
+        lk = county_lookup.get(self._head_location.county.upper(), None)
+
+        county_fips = str(0 if lk is None else lk['fips'])
+
+        org = "none" if lk is None else lk['regionOffice']
+
+        # print(county_fips)
+
         el = ElementTree.Element('fullEventUpdate')
 
         header = ElementTree.Element('message-header')
-        header.append(_get_org_info('organization-sending'))
+        header.append(_get_org_info('organization-sending', org))
         header.append(_el_tag_text('message-type-id', 'FEU'))
         header.append(_el_tag_text('message-type-version', '1'))
         header.append(_el_tag_text('message-number', '1'))
@@ -382,13 +474,13 @@ class EventUnplanned(_EventBase):
 
         element_description1 = ElementTree.Element('element-description')
         phrase1 = ElementTree.Element('phrase')
-        phrase1.append(_el_tag_text('accidentsAndIncidents', accidents_and_incidents))
+        phrase1.append(_el_tag_text('accidentsAndIncidents', self._description))
         element_description1.append(phrase1)
         element_descriptions.append(element_description1)
 
         element_description2 = ElementTree.Element('element-description')
         descrip1 = ElementTree.Element('additional-text')
-        descrip1.append(_el_tag_text('description', self._description))
+        descrip1.append(_el_tag_text('description', accidents_and_incidents))
         element_description2.append(descrip1)
         element_descriptions.append(element_description2)
 
@@ -414,6 +506,14 @@ class EventUnplanned(_EventBase):
 
         primary_location.append(_el_tag_text('link-name', self._head_location.roadway_name))
 
+        upward_area_reference = ElementTree.Element('upward-area-reference')
+        #TODO upward are reference is the fipscode
+
+        upward_area_reference.append(_el_tag_text('area-id', county_fips))
+        upward_area_reference.append(_el_tag_text('area-name', self._head_location.county.upper()))
+
+        primary_location.append(upward_area_reference)
+
         location_on_link.append(primary_location)
 
         if self._tail_location.latitude is not None and self._tail_location.longitude is not None:
@@ -731,6 +831,7 @@ class RouteStatus(Atms):
         return super().get(id_)
 
     def __init__(self, data: dict):
+
         super().__init__(data)
 
         self._calculated_tt: int = data['calculatedtt']
@@ -766,7 +867,7 @@ class RouteStatus(Atms):
     def xml_el(self):
         el = ElementTree.Element('routeData')
 
-        el.append(_get_org_info('organization-information'))
+        el.append(_get_org_info('organization-information', 'MILWAUKEE', 'Milwaukee Center'))
         el.append(_el_tag_text('route-id', self.id))
         el.append(_el_tag_text('route-status', '1'))
 
